#ifndef __MEDIAVIEW_H__
#define __MEDIAVIEW_H__

#include <QtGui>
#include <QtNetwork>
#include <phonon>
#include "View.h"
#include "ListModel.h"

class MediaView : public QWidget, public View {
    Q_OBJECT

public:
    MediaView(QWidget *parent);
    ~MediaView();

    // View
    void appear() {}
    void disappear();
    QMap<QString, QVariant> metadata() {
        QMap<QString, QVariant> metadata;
        metadata.insert("title", query);
        metadata.insert("description", tr("You're watching \"%1\"").arg(query));
        return metadata;
    }

    void setMediaObject(Phonon::MediaObject *mediaObject);

public slots:
    void setQuery(QString query);
    void pause();
    void stop();
    void skip();
    void fullscreen();
    void exitFullscreen();
    void openWebPage();
    void removeSelected();
    void moveUpSelected();
    void moveDownSelected();

private slots:
    // list/model
    void itemActivated(const QModelIndex &index);
    void selectionChanged (const QItemSelection & selected, const QItemSelection & deselected);
    void activeRowChanged(int);
    void selectVideos(QList<Video*> videos);
    // phonon
    void stateChanged(Phonon::State newState, Phonon::State oldState);
    void aboutToFinish();
    void currentSourceChanged(const Phonon::MediaSource source);
    // networkaccess
    void replyFinished();
    void metaDataChanged();

private:

    QString query;

    QSplitter *splitter;

    QListView *listView;
    ListModel *listModel;

    // phonon
    Phonon::MediaObject *mediaObject;
    Phonon::VideoWidget *videoWidget;

    QNetworkReply *networkReply;

};

#endif // __MEDIAVIEW_H__
