#include "updatechecker.h"
// #include "synchttp.h"
// #include "networkaccess.h"
#include "Constants.h"
#include <QtNetwork>

namespace The {
    QNetworkAccessManager* networkAccessManager();
}

UpdateChecker::UpdateChecker() {
    m_needUpdate = false;
}

void UpdateChecker::checkForUpdate() {
    QUrl updateUrl(QString(Constants::WEBSITE) + "-ws/release.xml");
    // QUrl updateUrl("http://flavio.tordini.org:8012/release.xml");

    QNetworkAccessManager *manager = The::networkAccessManager();
    networkReply = manager->get(QNetworkRequest(updateUrl));
    connect(networkReply, SIGNAL(metaDataChanged()),
            this, SLOT(metaDataChanged()), Qt::QueuedConnection);
}

void UpdateChecker::metaDataChanged() {

    QUrl redirection = networkReply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
    if (redirection.isValid()) {

        // qDebug() << "Redirect" << redirection;
        QNetworkAccessManager *manager = The::networkAccessManager();
        networkReply = manager->get(QNetworkRequest(redirection));
        connect(networkReply, SIGNAL(metaDataChanged()),
                this, SLOT(metaDataChanged()), Qt::QueuedConnection);

    } else {

        UpdateCheckerStreamReader reader;
        reader.read(networkReply->readAll());
        m_needUpdate = reader.needUpdate();
        m_remoteVersion = reader.remoteVersion();
        if (m_needUpdate) emit newVersion(m_remoteVersion);

    }

}

QString UpdateChecker::remoteVersion() {
    return m_remoteVersion;
}

// --- Reader ---

bool UpdateCheckerStreamReader::read(QByteArray data) {
    addData(data);

    while (!atEnd()) {
        readNext();
        if (isStartElement()) {
            if (name() == "release") {
                while (!atEnd()) {
                    readNext();
                    if (isStartElement() && name() == "version") {
                        QString remoteVersion = readElementText();
                        qDebug() << remoteVersion << QString(Constants::VERSION);
                        m_needUpdate = remoteVersion != QString(Constants::VERSION);
                        m_remoteVersion = remoteVersion;
                        break;
                    }
                }
            }
        }
    }

    return !error();
}

QString UpdateCheckerStreamReader::remoteVersion() {
    return m_remoteVersion;
}
