#include "youtubesearch.h"
#include "youtubestreamreader.h"
#include "Constants.h"

YouTubeSearch::YouTubeSearch() : QObject() {
    http = new QHttp();
    syncHttp = new SyncHttp(this);
}

void YouTubeSearch::request(QHttp *http, QUrl url) {
    QHttp::ConnectionMode mode = url.scheme().toLower() == "https" ?
                                 QHttp::ConnectionModeHttps : QHttp::ConnectionModeHttp;
    http->setHost(url.host(), mode, url.port() == -1 ? 0 : url.port());
    QHttpRequestHeader header("GET", url.path() + "?" + url.encodedQuery());
    header.setValue("Host", url.host());
    header.setValue("User-Agent", QString(Constants::APP_NAME) + " " +
                    Constants::VERSION + " (" + Constants::WEBSITE + ")");
    httpBuffer = new QBuffer(this);
    if (!httpBuffer->open(QBuffer::ReadWrite)) {
        qDebug("Cannot open buffer");
    }
    qDebug() << "GET " << url.toEncoded();
    httpRequestId = http->request(header, 0, httpBuffer);
}

void YouTubeSearch::search(QString query, int max, int skip) {
    this->abortFlag = false;
    http->disconnect(SIGNAL(requestFinished(int, bool)));
    connect(http, SIGNAL(requestFinished(int, bool)), this, SLOT(searchFinished(int, bool)));

    QString urlString = QString("http://gdata.youtube.com/feeds/api/videos?q=%1&max-results=%2&start-index=%3").arg(query, QString::number(max), QString::number(skip));
    // QString urlString = QString("http://localhost/oringo/video.xml?q=%1&max-results=%2&start-index=%3").arg(query, QString::number(max), QString::number(skip));
    QUrl url(urlString);
    request(http, url);
}

void YouTubeSearch::searchFinished(int requestId, bool error) {
    if (requestId != this->httpRequestId) return;

    if (abortFlag) {
        if (httpBuffer) delete httpBuffer;
        return;
    }

    if (error) {
        qDebug(http->errorString().toUtf8());
    } else {
        parseResults(httpBuffer);
    }

    if (httpBuffer) delete httpBuffer;
}

void YouTubeSearch::parseResults(QBuffer *buffer) {
    // qDebug() << buffer->data();
    YouTubeStreamReader reader;
    if (!reader.read(buffer)) {
        qDebug() << "Error while parsing XML";
    }
    videos = reader.getVideos();

    // and now for the dirty work...
    foreach (Video *video, videos) {

        video->setSyncHttp(syncHttp);

        // preload the thumb
        if (abortFlag) return;
        video->preloadThumbnail();

        // send it to the model
        emit gotVideo(video);
    }

    emit finished(videos.size());
}

QList<Video*> YouTubeSearch::getResults() {
    return videos;
}

void YouTubeSearch::abort() {
    this->abortFlag = true;
}
